var newVersion = '1.0.1';

/**
 * Converts the trigger entries from dsid to dsuid
 * @returns {boolean} true if property tree changed, false otherwize
 */
function convert2DSUID() {
    var fMigrate = false;
    try {
        lInfo.logln('convert2DSUID start');
        var iIndex, jIndex;
        var dSID, dSUID;
        var activityNode = Property.getNode('sonos-activities');
        var activityArray = activityNode.getChildren();
        for (iIndex = 0; iIndex < activityArray.length; iIndex++) {
            var triggerNode = activityArray[iIndex].getChild('triggers');
            var triggerArray = triggerNode.getChildren();
            for (jIndex = 0; jIndex < triggerArray.length; jIndex++) {
                switch(triggerArray[jIndex].getChild('type').getValue()) {
                    case 'device-msg': //intend fallthrough
                    case 'device-binary-input': //intend fallthrough
                    case 'device-scene': //intend fallthrough
                    case 'device-sensor': {
                        if (triggerArray[jIndex].getChild('dsid') != null) {
                            lInfo.logln('convert');    
                            dSID = triggerArray[jIndex].getChild('dsid').getValue();
                            dSUID = dSID2dSUID(dSID);
                            Property.setProperty(triggerArray[jIndex].getPath() + "/dsuid", dSUID);
                            Property.setFlag(triggerArray[jIndex].getPath() + "/dsuid", 'ARCHIVE', true);
                            triggerArray[jIndex].removeChild('dsid');
                            fMigrate = true;
                        }
                    }
                    break;
                    case 'state-change': {
                        var triggerName = triggerArray[jIndex].getChild('name').getValue();
                        var nameArray = triggerName.split('.');
                        if ((nameArray[0] === 'dev') && (nameArray[1].length === 24)) {
                            var dsuid = dSID2dSUID(nameArray[1]);
                            triggerName = '';
                            triggerName = triggerName.concat(nameArray[0],'.', dsuid, '.', nameArray[2]);
                            Property.setProperty(triggerArray[jIndex].getPath() + '/name', triggerName);  
                            fMigrate=true;
                        }
                    }
                    break;
                    default:{lInfo.logln('no convertion'); }
                }
            }
        }
    } catch(e) {
        lWarn.logln('Exception in backward-compatibility::convert2DSUID: ' + e);
    }
    return fMigrate;
}

/**
 * Compares version strings
 * @param {string} strV1 version string 1 to compare 
 * @param {string} strV2 version string 1 to compare
 * @returns {Number|undefined} return   number < 0  if strV1 < strV2
 *                                          0       if strV1 = strV2
 *                                      number >0   if strV1 < strV2
 */
function compVersions(strV1, strV2) {
    try {
        var nRes = 0;
        var parts1 = strV1.split('.');
        var parts2 = strV2.split('.');
        var nLen = Math.max(parts1.length, parts2.length);

        for (var i = 0; i < nLen; i++) {
          var nP1 = (i < parts1.length) ? parseInt(parts1[i], 10) : 0;
          var nP2 = (i < parts2.length) ? parseInt(parts2[i], 10) : 0;

          if (isNaN(nP1)) { nP1 = 0; }
          if (isNaN(nP2)) { nP2 = 0; }

          if (nP1 !== nP2) {
            nRes = (nP1 > nP2) ? 1 : -1;
            break;
          }
        }
        return nRes;
    } catch(e) {
        lWarn.logln('Exception in backward-compatibility::compVersions ' + e);
        return;       
    }
}

/**
 * convert property tree entries if needed
 * @returns {undefined}
 */
function convertPropertyTreeEntries()
{
    var fMigrated = false;
    try {      
        var propVersion = Property.getNode('version');
        var currentVersion = (propVersion != null) ? propVersion.getValue() : '0.0.0';
        lInfo.logln('check if conversions from version "' + currentVersion + '" to version "' + newVersion + '" are needed');    
        if (compVersions(currentVersion, '1.0.1') < 0) {
            fMigrated = convert2DSUID() || fMigrated;   
        } else {
            lInfo.logln('no conversion needed');    
        }
        Property.setProperty('version', newVersion);
        Property.setFlag('version', 'ARCHIVE', true);
        lInfo.logln("sonos-controller running (Version " + newVersion + ")");
    } catch (e) {
        lWarn.logln('Exception in backward-compatibility::convertPropertyTreeEntries ' + e);
    }
    if (fMigrated) {
        Property.store();
    }
}